package org.blue.club.utils;

import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import org.apache.commons.codec.binary.Base64OutputStream;
import org.blue.club.dao.redis.VerifyDao;
import org.blue.club.entities.dto.resp.VerifyCodeResponse;
import org.blue.club.entities.vo.VerifyVo;
import org.mmga.spring.boot.starter.utils.RandomUtils;
import org.springframework.stereotype.Component;

import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.nio.charset.StandardCharsets;
import java.util.Date;
import java.util.Random;

@Component
@RequiredArgsConstructor
public class VerifyCodeUtils {
    public static final int width = 120;
    public static final int height = 40;
    public static final int fontSize = 25;
    public static final Font mainFont = new Font("宋体", Font.BOLD, fontSize);

    private final RandomUtils randomUtils;
    private final VerifyDao verifyDao;

    private void setToRandomColor(Graphics g) {
        Random random = new Random();
        g.setColor(new Color(20 + random.nextInt(120), 20 + random.nextInt(120), 20 + random.nextInt(120)));
    }

    @SneakyThrows
    public VerifyCodeResponse generateVerifyCode() {
        BufferedImage image = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
        Graphics g = image.createGraphics();
        g.setFont(mainFont);
        String text = randomUtils.generatorRandomFileName(4);
        int x = 0;
        for (char c : text.toCharArray()) {
            Random random = new Random();
            setToRandomColor(g);
            g.drawString(c + "", 15 + x * 20, 20 + new Random().nextInt(10));
            for (int j = 0; j < random.nextInt(3, 7); j++) {
                setToRandomColor(g);
                g.drawLine(random.nextInt(width), random.nextInt(height), random.nextInt(width), random.nextInt(height));
            }
            x++;
        }
        g.dispose();
        String s = randomUtils.generatorRandomString(32);

        try (ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream(); Base64OutputStream base64OutputStream = new Base64OutputStream(byteArrayOutputStream)) {
            ImageIO.write(image, "png", base64OutputStream);
            base64OutputStream.flush();
            String base64 = byteArrayOutputStream.toString(StandardCharsets.UTF_8);
            verifyDao.save(new VerifyVo(s, text, new Date()));
            return new VerifyCodeResponse(base64, s);
        }
    }
}
